using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace Frog
{
	/// <summary>
	/// This is the main type for your game
	/// </summary>
	public class FrogGame : Microsoft.Xna.Framework.Game
	{
		//graphics
		GraphicsDeviceManager graphics;
		ContentManager content;
		public ContentManager Content { get { return content; } }
		public GraphicsDeviceManager Graphics { get { return graphics; } }
		public GraphicsDevice GraphicsDevice { get { return graphics.GraphicsDevice; } }
		SpriteBatch spriteBatch;
		public SpriteBatch SpriteBatch { get { return spriteBatch; } }

		public SpriteFont font;
		public Texture2D whitePixel;
		public Texture2D[,] dice;
		public Texture2D background;
		public Texture2D titleImage;
		public Texture2D padImage;
		public Texture2D beginImage;

		enum Screen
		{
			Title,
			Game
		}
		Screen mCurrentScreen = Screen.Title;
		TitleScreen mTitle;

		public Rectangle viewportRect;
		public Rectangle safeRect;

		public enum DiceTexture
		{
			One = 0, Two, Three, Four, Five, Six,
			White = 0, Red, Green, Blue
		};

		public Texture2D waterfall;
		public Effect refract;

		//game logic
		GameController controller;

		//input
		GamePadState lastPad, pad;
		KeyboardState lastKeyState, keyState;

		public enum Controls
		{
			RollDice,
			OK,
			AcceptMove,
			Up,
			Down,
			NoZonks,
			Start,
			Quit
		}

		public FrogGame()
		{
			graphics = new GraphicsDeviceManager(this);
			graphics.MinimumPixelShaderProfile = ShaderProfile.PS_2_0;
			content = new ContentManager(Services, "Content");
		}

		/// <summary>
		/// Allows the game to perform any initialization it needs to before starting to run.
		/// This is where it can query for any required services and load any non-graphic
		/// related content.  Calling base.Initialize will enumerate through any components
		/// and initialize them as well.
		/// </summary>
		protected override void Initialize()
		{
			lastPad = GamePad.GetState(PlayerIndex.One);
			lastKeyState = Keyboard.GetState();

			base.Initialize();
		}

		/// <summary>
		/// Load your graphics content.  If loadAllContent is true, you should
		/// load content from both ResourceManagementMode pools.  Otherwise, just
		/// load ResourceManagementMode.Manual content.
		/// </summary>
		/// <param name="loadAllContent">Which type of content to load.</param>
		protected override void LoadGraphicsContent(bool loadAllContent)
		{
			if (loadAllContent)
			{
				// TODO: Load any ResourceManagementMode.Automatic content
				LoadContent();
			}

			// TODO: Load any ResourceManagementMode.Manual content
		}

		/// <summary>
		/// LoadContent will be called once per game and is the place to load
		/// all of your content.
		/// </summary>
		//protected override void LoadContent() {
		protected void LoadContent()
		{
			updateSafeRect();

			// Create a new SpriteBatch, which can be used to draw textures.
			spriteBatch = new SpriteBatch(GraphicsDevice);

			font = this.Content.Load<SpriteFont>(@"Fonts\GameFont");
			whitePixel = this.Content.Load<Texture2D>(@"Sprites\whitepixel");
			background = this.Content.Load<Texture2D>(@"Sprites\jungle");

			RockSpace.rockSprite = this.Content.Load<Texture2D>(@"Sprites\rock_tile");
			LeafSpace.leafSprite = this.Content.Load<Texture2D>(@"Sprites\lilly_tile");
			InitialSpace.initialSprite = this.Content.Load<Texture2D>(@"Sprites\rock_tile");
			GoalSpace.goalSprite = this.Content.Load<Texture2D>(@"Sprites\goal_tile");
			Frog.frogSprite = this.Content.Load<Texture2D>(@"Sprites\Frogs\frog");
			titleImage = this.Content.Load<Texture2D>(@"Sprites\lily_txt");
			padImage = this.Content.Load<Texture2D>(@"Sprites\pad_txt");
			beginImage = this.Content.Load<Texture2D>(@"Sprites\beginImage");

			waterfall = this.Content.Load<Texture2D>(@"waterfall");
			refract = this.Content.Load<Effect>(@"Effects\refraction");

			GameBoard.background = this.Content.Load<Texture2D>(@"Sprites\water");


			dice = new Texture2D[6, 4];
			dice[(int)DiceTexture.One, (int)DiceTexture.White] = this.Content.Load<Texture2D>(@"Sprites\Dice\dice1_white");
			dice[(int)DiceTexture.One, (int)DiceTexture.Red] = this.Content.Load<Texture2D>(@"Sprites\Dice\dice1_red");
			for (int die = (int)DiceTexture.Two; die <= (int)DiceTexture.Six; die++)
			{
				dice[die, (int)DiceTexture.White] = this.Content.Load<Texture2D>(@"Sprites\Dice\dice" + (die + 1) + "_white");
				dice[die, (int)DiceTexture.Blue] = this.Content.Load<Texture2D>(@"Sprites\Dice\dice" + (die + 1) + "_blue");
				dice[die, (int)DiceTexture.Green] = this.Content.Load<Texture2D>(@"Sprites\Dice\dice" + (die + 1) + "_green");
			}

			for (int frog_die = 0; frog_die < 4; frog_die++)
			{
				Frog.frogDice[frog_die] = this.Content.Load<Texture2D>(@"Sprites\Frogs\frogs_" + (frog_die + 2));
			}
			Frog.frogLumen = this.Content.Load<Texture2D>(@"Sprites\Frogs\frog_lumen");

			controller = new GameController(this);
		}

		/// <summary>
		/// Unload your graphics content.  If unloadAllContent is true, you should
		/// unload content from both ResourceManagementMode pools.  Otherwise, just
		/// unload ResourceManagementMode.Manual content.  Manual content will get
		/// Disposed by the GraphicsDevice during a Reset.
		/// </summary>
		/// <param name="unloadAllContent">Which type of content to unload.</param>
		protected override void UnloadGraphicsContent(bool unloadAllContent)
		{
			if (unloadAllContent)
			{
				// TODO: Unload any ResourceManagementMode.Automatic content
				UnloadContent();
				content.Unload();
			}

			// TODO: Unload any ResourceManagementMode.Manual content
		}

		/// <summary>
		/// UnloadContent will be called once per game and is the place to unload
		/// all content.
		/// </summary>
		//        protected override void UnloadContent()
		protected void UnloadContent()
		{
			// TODO: Unload any non ContentManager content here
		}

		/// <summary>
		/// Allows the game to run logic such as updating the world,
		/// checking for collisions, gathering input, and playing audio.
		/// </summary>
		/// <param name="gameTime">Provides a snapshot of timing values.</param>
		protected override void Update(GameTime gameTime)
		{
			// Allows the game to exit
			pad = GamePad.GetState(PlayerIndex.One);
			keyState = Keyboard.GetState();
			if (pad.Buttons.Back == ButtonState.Pressed || keyState.IsKeyDown(Keys.Escape))
				this.Exit();

			switch (mCurrentScreen)
			{
				case Screen.Title:
					{
						//Create a new title screen object if
						//one hasn't already been created
						if (mTitle == null)
						{
							mTitle = new TitleScreen(this);
						}

						//Update the Title screen objects
						mTitle.Update(this, gameTime);

						//Check to see if it's time to start
						//the game
						if (mTitle.StartGame == true)
						{
							mCurrentScreen = Screen.Game;
						}
						break;
					}

				case Screen.Game:
					{
						switch (controller.State)
						{
							case GameController.GameState.NewTurn:
								if (CheckControl(Controls.RollDice, ButtonState.Pressed))
									controller.RollDice();
								break;
							case GameController.GameState.PickFrog:
								CheckControl(Controls.NoZonks, ButtonState.Pressed);
								if (CheckControl(Controls.OK, ButtonState.Pressed))
									controller.PickFrog();
								else if (CheckControl(Controls.Up, ButtonState.Pressed))
									controller.IncrementLaneChoice(-1);
								else if (CheckControl(Controls.Down, ButtonState.Pressed))
									controller.IncrementLaneChoice(1);
								break;
							case GameController.GameState.TryMove:
								if (CheckControl(Controls.RollDice, ButtonState.Pressed))
									controller.RollDice();
								else if (CheckControl(Controls.AcceptMove, ButtonState.Pressed))
									controller.AcceptMove();
								break;
							case GameController.GameState.Zonked:
							case GameController.GameState.FrogFinish:
								if (CheckControl(Controls.OK, ButtonState.Pressed))
									controller.AcceptMove();
								break;
							case GameController.GameState.GameOver:
								if (CheckControl(Controls.Start, ButtonState.Pressed))
									controller.NewGame();
								break;
						}

						lastPad = pad;
						lastKeyState = keyState;
						break;
					}
			}

			base.Update(gameTime);
		}

		public bool CheckControl(Controls control, ButtonState pressed)
		{
			return CheckXboxControl(control, pressed) || CheckKeyboardControl(control, pressed);
		}

		private bool CheckXboxControl(Controls control, ButtonState pressed)
		{
			ButtonState l, c;
			switch (control)
			{
				case Controls.AcceptMove:
					l = lastPad.Buttons.B;
					c = pad.Buttons.B;
					break;
				case Controls.OK:
				case Controls.RollDice:
					l = lastPad.Buttons.A;
					c = pad.Buttons.A;
					break;
				case Controls.Quit:
					l = lastPad.Buttons.Back;
					c = pad.Buttons.Back;
					break;
				case Controls.Up:
					if (pressed == ButtonState.Pressed)
					{
						if (pad.ThumbSticks.Left.Y > 0.9f && lastPad.ThumbSticks.Left.Y <= 0.9f)
							return true;
					}
					else
					{
						if (pad.ThumbSticks.Left.Y <= 0.9f && lastPad.ThumbSticks.Left.Y > 0.9f)
							return true;
					}
					//Joysticks didn't report anything interesting, try D-pad
					l = lastPad.DPad.Up;
					c = pad.DPad.Up;
					break;
				case Controls.Down:
					if (pressed == ButtonState.Pressed)
					{
						if (pad.ThumbSticks.Left.Y < -0.9f && lastPad.ThumbSticks.Left.Y >= -0.9f)
							return true;
					}
					else
					{
						if (pad.ThumbSticks.Left.Y >= -0.9f && lastPad.ThumbSticks.Left.Y < -0.9f)
							return true;
					}
					l = lastPad.DPad.Down;
					c = pad.DPad.Down;
					break;
				case Controls.Start:
					l = lastPad.Buttons.Start;
					c = pad.Buttons.Start;
					break;
				case Controls.NoZonks:
					return pad.Triggers.Left > 0.9f;
				default:
					return false;
			}
			if (pressed == ButtonState.Pressed)
				return (c == ButtonState.Pressed && l == ButtonState.Released);
			else
				return (c == ButtonState.Released && l == ButtonState.Pressed);
		}

		private bool CheckKeyboardControl(Controls control, ButtonState pressed)
		{
			Keys key;
			switch (control)
			{
				case Controls.AcceptMove:
					key = Keys.Space;
					break;
				case Controls.OK:
				case Controls.RollDice:
					key = Keys.Enter;
					break;
				case Controls.Quit:
					key = Keys.Escape;
					break;
				case Controls.Up:
					key = Keys.Up;
					break;
				case Controls.Down:
					key = Keys.Down;
					break;
				case Controls.Start:
					key = Keys.Enter;
					break;
				case Controls.NoZonks:
					return keyState.IsKeyDown(Keys.Z);
				default:
					return false;
			}
			if (pressed == ButtonState.Pressed)
				return (keyState.IsKeyDown(key) && lastKeyState.IsKeyUp(key));
			else
				return (keyState.IsKeyUp(key) && lastKeyState.IsKeyDown(key));
		}

		/// <summary>
		/// This is called when the game should draw itself.
		/// </summary>
		/// <param name="gameTime">Provides a snapshot of timing values.</param>
		protected override void Draw(GameTime gameTime)
		{
			graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

			switch (mCurrentScreen)
			{
				case Screen.Title:
					{
						//Draw the Title screen objects
						mTitle.Draw(this, gameTime);
						break;
					}

				case Screen.Game:
					{
						updateSafeRect();

						spriteBatch.Begin();
						spriteBatch.Draw(background, viewportRect, Color.White);
						spriteBatch.End();

						controller.Draw(this, gameTime);
						break;
					}
			}
			base.Draw(gameTime);
		}

		public void DrawBorder(Rectangle rect, int width, Color bg)
		{
			spriteBatch.Draw(whitePixel, rect, bg);
			spriteBatch.Draw(whitePixel, new Rectangle(rect.Left, rect.Top, rect.Width, width), Color.Black);
			spriteBatch.Draw(whitePixel, new Rectangle(rect.Left, rect.Bottom - width, rect.Width, width), Color.Black);
			spriteBatch.Draw(whitePixel, new Rectangle(rect.Left, rect.Top, width, rect.Height), Color.Black);
			spriteBatch.Draw(whitePixel, new Rectangle(rect.Right - width, rect.Top, width, rect.Height), Color.Black);
		}

		private void updateSafeRect()
		{
			Viewport viewport = graphics.GraphicsDevice.Viewport;
			viewportRect = new Rectangle(0, 0, viewport.Width, viewport.Height);
#if XBOX
			safeRect = new Rectangle((int)(viewport.Width * 0.05),
									 (int)(viewport.Height * 0.05),
									 (int)(viewport.Width * 0.9),
									 (int)(viewport.Height * 0.9));
#else
			safeRect = viewportRect;
#endif
		}

		private const double WaveSpeed = 0.08f;
		public void DrawWater(Rectangle rect, GameTime time)
		{
			double t = time.TotalGameTime.TotalSeconds * WaveSpeed;

			SpriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Immediate, SaveStateMode.None);
			GraphicsDevice.Textures[1] = waterfall;
			refract.Parameters["DisplacementScroll"].SetValue(new Vector2((float)Math.Cos(t), (float)Math.Sin(t)));
			refract.Begin();
			refract.CurrentTechnique.Passes[0].Begin();
			SpriteBatch.Draw(GameBoard.background, rect, Color.White);
			SpriteBatch.End();
			refract.CurrentTechnique.Passes[0].End();
			refract.End();
		}
	}
}
