using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace Frog {
	class GameBoard {
		private List<GameSpace>[] board;
		private int columns;
        public static Texture2D background;

		//private const int SpaceWidth = 64;
		//private const int SpaceHeight = 64;

		private const int BorderWidth = 5;

		public GameBoard(Random random, int rows, int cols) {
			board = new List<GameSpace>[rows];
			columns = cols+3;
			for(int i = 0; i < rows; i++) {
				board[i] = new List<GameSpace>();
				board[i].Add(new InitialSpace(0));
				for(int j = 1; j <= cols; j++) {
					int rand = random.Next(10);
					GameSpace space;
					if(rand < 3)
						space = new LeafSpace(j);
					else if(rand < 5)
						space = new RockSpace(j);
					else
						space = new WaterSpace(j);
					board[i].Add(space);
				}
				board[i].Add(new GoalSpace(cols + 1));
			}
		}

		public List<GameSpace> this[int i]
		{
			get { return board[i]; }
		}

		public int LaneCount { get { return board.Length; } }

        public bool Move(Frog f, int space)
        {
            List<GameSpace> lane = board[f.ID];
            
			if(space >= lane.Count-1)
			{
                FinishFrog(f);
                //Accept();
				return true;
			}
			else if (space >= 0)
            {
                   lane[space].Hold(this, f);
            }
			return false;
        }

		public void FinishFrog(Frog f)
		{
            List<GameSpace> lane = board[f.ID];
            f.finish();  // set the frog as finished

            //extend the board if frog has reached end and there is no GoalSpace - this is the first frog to finish
            if (lane.Count == GameController.InitialSpacesPerLane+2)
            {
                lane[lane.Count-1].SpaceNumber = lane.Count;
                lane.Insert(lane.Count - 1, new WaterSpace(lane.Count - 1));
            }
		}

        public void Accept()
        {
            foreach (List<GameSpace> lane in board)
            {
                foreach (GameSpace space in lane)
                {
                    space.Accept();
                }
            }
        }

        public void Forget()
        {
            foreach (List<GameSpace> lane in board) {
                foreach (GameSpace space in lane)
                {
                    space.Forget();
                }
            }
        }

		public void Draw(FrogGame game, Rectangle rect, GameTime time) {
			game.SpriteBatch.Begin();
			game.DrawBorder(rect, BorderWidth, Color.Black);
			game.SpriteBatch.End();

			Rectangle gameRect = new Rectangle(rect.Left + BorderWidth, rect.Top + BorderWidth, rect.Width - (BorderWidth * 2), rect.Height - (BorderWidth * 2));
			int SpaceWidth = gameRect.Width / columns;
			int SpaceHeight = gameRect.Height / board.Length;

			/*game.SpriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Immediate, SaveStateMode.None);
			game.GraphicsDevice.Textures[1] = game.waterfall;
			game.refract.Parameters["DisplacementScroll"].SetValue(new Vector2((float)Math.Cos(t), (float)Math.Sin(t)));
			game.refract.Begin();
			game.refract.CurrentTechnique.Passes[0].Begin();
            game.SpriteBatch.Draw(background, gameRect, Color.White);
			game.SpriteBatch.End();
			game.refract.CurrentTechnique.Passes[0].End();
			game.refract.End();*/
			game.DrawWater(gameRect, time);

			Color laneColor = new Color(0,0,0,32);

			game.SpriteBatch.Begin();
			for(int i = 0; i < board.Length; i++) {
				if(i > 0)
					game.SpriteBatch.Draw(game.whitePixel, new Rectangle(rect.Left, i * SpaceHeight + gameRect.Top, rect.Width, 2), laneColor);
				for(int j = 0; j < board[i].Count; j++) {
					Rectangle spaceRect = new Rectangle(j * SpaceWidth + gameRect.Left, i * SpaceHeight + gameRect.Top, SpaceWidth, SpaceHeight);
					if(j < board[i].Count-1)
						game.SpriteBatch.Draw(game.whitePixel, new Rectangle(spaceRect.Right, spaceRect.Top, 2, spaceRect.Height), laneColor);
					board[i][j].Draw(game, spaceRect);
				}
			}
			game.SpriteBatch.End();
		}
	}
}
