using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace Frog
{
	abstract class GameSpace
	{
		protected Frog currVisitor;  // the current frog sitting on the space
		protected Frog virtualVisitor;  // when doing simulated moves, space is holding this virtual frog
		protected int spaceNumber;  // ordinal number marking space's position in relation to other spaces and the end space
		protected Texture2D sprite;  // the picture of this space

		protected GameSpace(int spaceID)
		{
			currVisitor = null;
			virtualVisitor = null;
			spaceNumber = spaceID;
			sprite = null;
		}

		//abstract methods
		public abstract void Hold(GameBoard board, Frog incomingFrog);  //simulates a frog move when moves have not accepted

		public virtual void Remove(Frog frog)
		{
			if(currVisitor == frog)
				virtualVisitor = null;
		}

		//sets the frog at its current location
		public virtual void Accept()
		{
			currVisitor = virtualVisitor;
			if(currVisitor != null)
				currVisitor.Space = this;
		}
		//clears out the virtual move data since move not accepted
		public virtual void Forget()
		{
			virtualVisitor = currVisitor;
		}

		public Frog CurrVisitor
		{
			get { return currVisitor; }
			set { currVisitor = value; }
		}

		public int SpaceNumber
		{
			get { return spaceNumber; }
			set { spaceNumber = value; }
		}

		public virtual void Draw(FrogGame game, Rectangle rect)
		{
            // Draw the tile for this game space, if any.
            if (sprite != null)
            {
                game.SpriteBatch.Draw(sprite, rect, Color.White);
            }

            // Draw the visiting frog, if any.
            if (virtualVisitor != null)
            {
				/*int xoffset = rect.Width / 8;
				int yoffset = rect.Height / 8;
				int rwidth = rect.Width * 3 / 4;
				int rheight = rect.Height * 3 / 4;

				Rectangle frogRect = new Rectangle(rect.X + xoffset, rect.Y + yoffset, rwidth, rheight);*/
				virtualVisitor.Draw(game, rect);
            }
		}
	}
}
