using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace Frog
{
	class InitialSpace : GameSpace
	{
		public static Texture2D initialSprite;

		private List<Frog> frogs;
		private List<Frog> virtualFrogs;

		public InitialSpace(int spaceID)
			: base(spaceID)
		{
			sprite = initialSprite;
			frogs = new List<Frog>();
			virtualFrogs = new List<Frog>();
		}

		public override void Accept()
		{
			frogs.Clear();
			frogs.AddRange(virtualFrogs);
            if (frogs.Count > 0)
            {
                frogs[frogs.Count - 1].Space = this;
            }
		}

		public override void Forget()
		{
			virtualFrogs.Clear();
			virtualFrogs.AddRange(frogs);
		}

		public override void Hold(GameBoard board, Frog incomingFrog)
		{
			virtualFrogs.Add(incomingFrog);
		}

		public override void Remove(Frog frog)
		{
			virtualFrogs.Remove(frog);
		}

		public override void Draw(FrogGame game, Rectangle rect)
		{
            // Draw the initial space tile, if any.
            if (sprite != null)
            {
                int rW = rect.Width / 2;
                int rH = rect.Height / 2;

                for (int i = 0; i < 4; i++)
                {
                    int rX = rect.X + (i % 2) * rW;
                    int rY = rect.Y + (i / 2) * rH;

                    Rectangle cornerBox = new Rectangle(rX, rY, rW, rH);
                    game.SpriteBatch.Draw(sprite, cornerBox, Color.White);
                }
                game.SpriteBatch.Draw(sprite, rect, Color.White);
            }

            if (virtualFrogs.Count > 0)
            {
                int partition = (int)Math.Sqrt(virtualFrogs.Count);
                if (partition * partition < virtualFrogs.Count)
                    partition++;

                int rWidth = rect.Width / partition;
                int rHeight = rect.Height / partition;
                int xoffset = 0;
                int yoffset = 0;

                foreach (Frog f in virtualFrogs)
                {
                    Rectangle frogRect = new Rectangle(rect.X + xoffset, rect.Y + yoffset, rWidth, rHeight);
                    f.Draw(game, frogRect);

                    xoffset += rWidth;

                    if (xoffset + rWidth > rect.Width)
                    {
                        xoffset = 0;
                        yoffset += rHeight;
                    }
                }
            }
            
		}
	}
}
