using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace Frog
{
    class TitleScreen
    {
        int mPositionX;
        int mPositionY;
        int mPadPosX = 100;
        int mPadPosY = 150;
	    double mScale = .50;

        Rectangle mTitleRectangle;
        Rectangle mPadRectangle;
        //Rectangle mStartRectangle;
		String str = "Press Start (Enter) to begin";
		Vector2 strPos;

        double mTime = .5f;   
        
        private enum TitleState
        {
            Falling,
            BouncingAway,
            BouncingTowards,
            Still
        }
        TitleState mCurrentState = TitleState.Falling;

        private bool mStartGame = false;
        public bool StartGame
        {
            get { return mStartGame; }
        }

        public TitleScreen(FrogGame theGame)
        {
            theGame.GraphicsDevice.PresentationParameters.BackBufferFormat = SurfaceFormat.Bgr24;

			mPositionX = (int)(theGame.safeRect.Width - (theGame.titleImage.Width * mScale)) / 2 - 60 + theGame.safeRect.Left;

            mPositionY = theGame.titleImage.Height * -1;
            mPadPosY = 600 + (int)(theGame.padImage.Height * mScale) - 10;

            mTitleRectangle = new Rectangle(mPositionX, mPositionY, (int)(theGame.titleImage.Width * mScale), (int)(theGame.titleImage.Height * mScale));
            mPadRectangle = new Rectangle(mPadPosX + 120, mPadPosY, (int)(theGame.padImage.Width * mScale), (int)(theGame.titleImage.Height * mScale));
            //mStartRectangle = new Rectangle(300, 350, (int)(theGame.titleImage.Width * mScale), (int)(theGame.titleImage.Height * mScale));
			
			Vector2 vec = theGame.font.MeasureString(str);
			strPos = new Vector2((theGame.safeRect.Width - vec.X) / 2 + theGame.safeRect.Left, theGame.viewportRect.Height * 0.6f);

            theGame.GraphicsDevice.Clear(Color.Black);        
        }

        public void Update(FrogGame theGame, GameTime theGameTime)
        {
            mTime -= theGameTime.ElapsedGameTime.Milliseconds;
            switch (mCurrentState)
            {
                case TitleState.Falling:
                    {
                        if (mTime < 0)
                        {
                            mTime = 1.5f;
                            mPositionY += 3;
                            mPadPosY -= 4;
                        }

                        if (mPositionY > 145)
                        {
                            mCurrentState = TitleState.BouncingAway;
                        }

                        break;
                    }

                case TitleState.BouncingAway:
                    {
                        if (mTime < 0)
                        {
                            mTime = 2.0f;
                            mPositionY -= 2;
                            mPadPosY += 4;
                        }

                        if (mPositionY < 130)
                        {
                            mCurrentState = TitleState.BouncingTowards;
                        }

                        break;
                    }

                case TitleState.BouncingTowards:
                    {
                        if (mTime < 0)
                        {
                            mTime = 4.5f;
                            mPositionY += 1;
                            mPadPosY -= 2;
                        }

                        if (mPositionY > 145)
                        {
                            mCurrentState = TitleState.Still;
                        }

                        break;
                    }
            }
            
            mTitleRectangle = new Rectangle(mPositionX, mPositionY, (int)(theGame.titleImage.Width * mScale), (int)(theGame.titleImage.Height * mScale));
            mPadRectangle = new Rectangle(mPositionX + 120, mPadPosY, (int)(theGame.padImage.Width * mScale), (int)(theGame.padImage.Height * mScale));

            //KeyboardState keyState = Keyboard.GetState();
			//GamePadState pad = GamePad.GetState(PlayerIndex.One);
            //if (keyState.IsKeyDown(Keys.F5) || pad.Buttons.Start == ButtonState.Pressed)
			if(theGame.CheckControl(FrogGame.Controls.Start, ButtonState.Pressed))
            {
                if (mCurrentState == TitleState.Still)
                {
                    mStartGame = true;
                }
            }
        }

        public void Draw(FrogGame theGame, GameTime theGameTime)
        {
            SpriteBatch sb = theGame.SpriteBatch;

			theGame.DrawWater(theGame.viewportRect, theGameTime);

            sb.Begin(); //mbatch is the spritebatch

            //sb.Draw(feedbackTexture, new Rectangle(0, 0, (int)(feedbackTexture.Width * 1), (int)(feedbackTexture.Height * 1)), new Color(255, 255, 255, 227));
            sb.Draw(theGame.titleImage, mTitleRectangle, Color.White);
            sb.Draw(theGame.padImage, mPadRectangle, Color.White);


            if (mCurrentState == TitleState.Still)
            {
                //sb.Draw(theGame.beginImage, mStartRectangle, Color.White);
                byte alpha = (byte)(Math.Cos(theGameTime.TotalGameTime.TotalSeconds * 2)*64+191);
				sb.DrawString(theGame.font, str, strPos, new Color(0,0,0,alpha));
            }

			//theGame.DrawBorder(theGame.safeRect, 4, new Color(0, 0, 0, 64));

            sb.End();

            //theGame.GraphicsDevice.ResolveBackBuffer(feedbackTexture);
        }

    }
}
